//*****************************************************************************
//                                CmdLinePcr.cpp                              *
//                               ----------------                             *
//  Started     : 22/02/2005                                                  *
//  Last Update : 23/06/2005                                                  *
//  Copyright   : (C) 2003 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "CmdLinePcr.hpp"

//*****************************************************************************
// Constructor.
//
// Arguments:
//   poApp - A pointer to the AppGSpiceUI object

CmdLinePcr::CmdLinePcr( AppGSpiceUI * poApp )
{
  if( poApp != NULL )
  {
    m_poApp      = poApp;
    m_poFrmMain  = poApp->m_poFrmMain;
    m_poFileTsks = &(poApp->m_poFrmMain->m_oFileTsks);
  }
  else
  {
    m_poApp      = NULL;
    m_poFrmMain  = NULL;
    m_poFileTsks = NULL;
  }
  m_iArgC   = 0;
  m_ppsArgV = NULL;
}

//*****************************************************************************
// Destructor.

CmdLinePcr::~CmdLinePcr( )
{
}

//*****************************************************************************
// Process command line option -a : analysis page specifier.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_a( int * piArg )
{
  ( *piArg )++;

  if( *piArg >= m_iArgC )
  {
    std::cout << "Missing analysis type specifier.\n";
    return( FALSE );
  }

  if( ! m_poFrmMain->m_poNbkSimr->bSetPage( m_ppsArgV[ *piArg ] ) )
  {
    std::cout << "Analysis type invalid or not supported by "
              << m_poFrmMain->m_poPrcSimr->roGetBinary( ).GetFullName( ).mb_str( )
              << " : " << m_ppsArgV[ *piArg ] << '\n';
    return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Process command line option -c : specify a SPICE command file.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_c( int * piArg )
{
  wxString  os1;

  std::cout << "Option -c is not yet implemented\n";

  return( FALSE );
}

//*****************************************************************************
// Process command line option -g : Guile procedure used when importing
//                                  schematic file.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_g( int * piArg )
{
  wxString  os1;

  // Increment argument pointer
  ( *piArg )++;

  // Check that there are more arguments and that the next is not an option
  if( *piArg<(m_iArgC-1) && *m_ppsArgV[*piArg]!='-' )
  {
    os1 = m_ppsArgV[ *piArg ];
    if( m_poFileTsks->bSetGuileProc( os1 ) ) return( TRUE );

    // Attempt to determine if next argument is a file name
    wxString  os1( m_ppsArgV[ *piArg ] );
    wxRegEx   oRegEx( wxT("[./]") );
    if( os1.Length( )>0 && !oRegEx.Matches( os1 ) )
    {
      std::cout << "Invalid Guile procedure name : "
                << m_ppsArgV[ *piArg ] << '\n';
      return( FALSE );
    }
  }

  // Use the default Guile procedure name
  os1 = wxT("spice-sdb");
  m_poFileTsks->bSetGuileProc( os1 );
  ( *piArg )--;

  return( TRUE );
}

//*****************************************************************************
// Process command line option -h : display usage and exit.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_h( int * piArg )
{
  Usage( );

  return( FALSE );
}

//*****************************************************************************
// Process command line option -l : maximum lines allowed in text controls.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_l( int * piArg )
{
  wxString  osArg;
  long      l1;

  ( *piArg )++;

  if( *piArg >= m_iArgC )
  {
    std::cout << "Missing maximum line count value.\n";
    return( FALSE );
  }

  osArg = wxString( m_ppsArgV[ *piArg ] ).Upper( );

  if( ! osArg.ToLong( &l1 ) )
  {
    std::cout << "Invalid maximum line count value : " << osArg.mb_str( ) << '\n';
    return( FALSE );
  }

  if( ! m_poFrmMain->m_oNbkTxtCtls.bSetLinesMax( (int) l1 ) )
  {
    std::cout << "Maximum line count value out of range ("
              << TXTCTL_LINESMIN << " to " << TXTCTL_LINESMAX << ")\n";
    return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Process command line option -d : specify the number of lines to display in
//                                  text controls.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_d( int * piArg )
{
  wxString  osArg;
  long      l1;

  ( *piArg )++;

  if( *piArg >= m_iArgC )
  {
    std::cout << "Missing display lines value.\n";
    return( FALSE );
  }

  osArg = wxString( m_ppsArgV[ *piArg ] ).Upper( );

  if( ! osArg.ToLong( &l1 ) )
  {
    std::cout << "Invalid display lines value : " << osArg.mb_str( ) << '\n';
    return( FALSE );
  }

  if( l1<TXTCTL_DISPMIN || l1>TXTCTL_DISPMAX )
  {
    std::cout << "Display lines value out of range ("
              << TXTCTL_DISPMIN << " to " << TXTCTL_DISPMAX << ")\n";
    return( FALSE );
  }

  TextCtrl::bSetLinesDsp( (int) l1 );
  l1 = (long) TextCtrl::iGetPixelHt( ) + TXTCTL_NBKTABHT;
  m_poFrmMain->SetClientSize( -1, 260 + l1 );
  m_poFrmMain->m_oNbkTxtCtls.SetClientSize( -1, l1 );

  return( TRUE );
}

//*****************************************************************************
// Process command line option -r : set the temporary file management strategy.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_t( int * piArg )
{
  std::cout << "Option -t is not yet implemented\n";

  return( FALSE );
}

//*****************************************************************************
// Process command line option -r : specify a configuration file.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_r( int * piArg )
{
  std::cout << "Option -r is not yet implemented\n";

  return( FALSE );
}

//*****************************************************************************
// Process command line option -s : simulator engine specifier.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_s( int * piArg )
{
  wxString  osArg;

  ( *piArg )++;

  if( *piArg >= m_iArgC )
  {
    std::cout << "Missing simulator engine specifier.\n";
    return( FALSE );
  }

  osArg = wxString( m_ppsArgV[ *piArg ] ).Upper( );

  if(      osArg == wxT("GNUCAP")  )
    m_poFrmMain->InitSimrEng( Simulation::eSIMR_GNUCAP );
  else if( osArg == wxT("NGSPICE") )
    m_poFrmMain->InitSimrEng( Simulation::eSIMR_NGSPICE );
  else
  {
    std::cout << "Invalid simulator engine specifier : "
              << m_ppsArgV[ *piArg ] << '\n';
    return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Process command line option -v : display app. version and exit.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcOption_v( int * piArg )
{
  wxString  os1;

  os1 << APP_NAME << wxT(", ") << APP_VERSION << wxT('\n');
  std::cout << os1.mb_str( );

  return( FALSE );
}

//*****************************************************************************
// Process net list or schematic file name command line argument.
//
// Argument List:
//   piArg - A pointer to the current command line argument
//           (incremented as part of processing)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bProcFileName( int * piArg )
{
  wxArrayString  oasFNames;
  wxFileName     oFName;
  wxString  os1;
  size_t  szt1;

  // Check that there are enough arguments
  if( *piArg >= m_iArgC )
  {
    std::cout << "Missing schematic file name/s.\n";
    return( FALSE );
  }

  // Check the validity of the file name/s
  while( *piArg < m_iArgC )
  {
    oFName = wxString( m_ppsArgV[ *piArg ], *wxConvCurrent );
    if( ! oFName.IsOk( ) )
    {
      std::cout << "Invalid file name : " << m_ppsArgV[ *piArg ] << '\n';
      return( FALSE );
    }
    if( ! oFName.FileExists( ) )
    {
      std::cout << "File doesn't exist : " << m_ppsArgV[ *piArg ] << '\n';
      return( FALSE );
    }
    if( oFName.IsRelative( ) ) oFName.MakeAbsolute( );
    oasFNames.Add( oFName.GetFullPath( ) );
    (*piArg)++;
  }

  // Import schematic file/s if specified
  if( ! m_poFileTsks->rosGetGuileProc( ).IsEmpty( ) )
  {
    // Check that gnetlist exists and is accessible
    if( ! m_poFileTsks->bOkGNetList( ) )
    {
      std::cout << "GNetList required to import schematic file/s wasn't found\n";
      return( FALSE );
    }

    m_poFileTsks->bSetSchems( oasFNames );

    if( ! m_poFileTsks->bReload( m_poFrmMain->m_oSimn ) )
    {
      os1 << wxT("Couldn't import schematic file/s : ");
      for( szt1=0; szt1<oasFNames.GetCount( ); szt1++ )
        os1 << oasFNames.Item( szt1 );
      std::cout << os1.mb_str( ) << '\n';
      return( FALSE );
    }
  }
  else
  {
    if( oasFNames.GetCount( ) > 1 )
    {
      os1 << wxT("Can't load multiple net list files : ");
      for( szt1=0; szt1<oasFNames.GetCount( ); szt1++ )
        os1 << oasFNames.Item( szt1 );
      std::cout << os1.mb_str( ) << '\n';
      return( FALSE );
    }
    m_poFileTsks->bSetNetList( oFName.GetFullPath( ) );
  }

  // Load the net list file
  if( m_poFileTsks->bReload( m_poFrmMain->m_oSimn ) )
  {
    if( m_poFrmMain->bLoad( ) )
    {
      m_poFrmMain->m_oNbkTxtCtls.bSetPage( NbkTxtCtls::ePAGE_NETLIST );
      return( TRUE );
    }
  }

  std::cout << "Couldn't load net list file : "
            << m_poFileTsks->rosGetNetList( ).mb_str( ) << '\n';

  return( FALSE );
}

//*****************************************************************************
// Print version information on console.

void  CmdLinePcr::Version( void )
{
  wxString  os1;

  os1 << wxT("\n            ") << APP_NAME
      << wxT("\n  ") << APP_VERSION << wxT("\n\n");

  std::cout << os1.mb_str( );
}

//*****************************************************************************
// Print usage message on console.

void  CmdLinePcr::Usage( void )
{
  wxString  os1;

  os1 << wxT("\nUsage : ") << m_poApp->GetAppName( ) << wxT(" [-OPTION [ARG]] [FILE/S]\n")
      << wxT("Analyse a circuit using a SPICE electronic circuit simulation.\n")
      << wxT("Options :\n")
      << wxT("  -v        : Print version information.\n")
      << wxT("  -h        : Print usage (this message).\n")
      << wxT("  -l LINES  : Specify the maximum line count allowed in text controls.\n")
      << wxT("                LINES = ") << TXTCTL_LINESMIN << wxT(" to ")
      << TXTCTL_LINESMAX << wxT(" (default = ") << TXTCTL_LINESDEF << wxT(")\n")
      << wxT("  -d LINES  : Specify the number of lines to display in text controls.\n")
      << wxT("                LINES = ") << TXTCTL_DISPMIN << wxT(" to ")
      << TXTCTL_DISPMAX << wxT(" (default = ") << TXTCTL_DISPDEF << wxT(")\n")
      << wxT("  -t VALUE  : Temporary file management strategy.\n")
      << wxT("                VALUE = 0 delete (default), 1 prompt, 2 keep\n")
//    << wxT("  -r RCFILE  : Specify a configuration file eg. .gspiceuirc.\n")
//    << wxT("  -c CMDFILE : Specify a SPICE command file.\n")
      << wxT("  -s SPEC   : Specify the simulation engine to be used.\n")
      << wxT("                SPEC = gnucap (default) or ngspice\n")
      << wxT("  -a TYPE   : Specify the analysis page to be displayed.\n")
      << wxT("                TYPE = OP, DC (default), AC, TR, FO, DI, NO, PZ, SE or TF\n")
      << wxT("  -g [PROC] : Guile procedure to import a schematic file using GNetList.\n")
      << wxT("                PROC = spice-sdb (default), etc.\n")
      << wxT("  FILE/S    : Load a circuit description file or import schematic file/s.\n\n");

  std::cout << os1.mb_str( );
}

//*****************************************************************************
// Set the command line to be processed.
//
// Arguments:
//   iArgV   - The argment count
//   ppsArgV - The string array of argument fields
//
// Return Arguments:
//   TRUE  - Success
//   FALSE - Failure

bool  CmdLinePcr::bSetCmdLine( int iArgC, wxChar ** ppsArgV )
{
  // Check validity of arguments
  if( iArgC <= 0 )      return( FALSE );
  if( ppsArgV == NULL ) return( FALSE );

  m_iArgC   = iArgC;
  m_ppsArgV = ppsArgV;

  return( TRUE );
}

//*****************************************************************************
// Process any command line arguments past to the application at start-up.
//
// Return Arguments:
//   TRUE  - Success (continue  application execution)
//   FALSE - Failure (terminate application execution)

bool  CmdLinePcr::bProcArgs( void )
{
  int  iArg;
  wxString  osArg, os1;

  // Check validity of arguments
  if( m_iArgC <= 0 )         return( FALSE );
  if( *m_ppsArgV   == NULL ) return( FALSE );
  if( m_poApp      == NULL ) return( FALSE );
  if( m_poFrmMain  == NULL ) return( FALSE );
  if( m_poFileTsks == NULL ) return( FALSE );

  // Process options
  for( iArg=1, osArg=m_ppsArgV[1]; iArg<m_iArgC; ++iArg, osArg=m_ppsArgV[iArg] )
  {
    if( osArg.GetChar( 0 )=='-' && osArg.Length( )>2 )
    { // Concatenated options
      std::cout << "Options must be specfied separately : " << osArg.mb_str( ) << '\n';
      return( FALSE );
    } 
    else if( osArg == wxT("-v") )
    { // Print version information
      if( ! bProcOption_v( &iArg ) ) return( FALSE );
    }
    else if( osArg == wxT("-h") )
    { // Print usage information
      if( ! bProcOption_h( &iArg ) ) return( FALSE );
    }
    else if( osArg == wxT("-l") )
    { // Specify the maximum number of lines allowed in the text controls
      if( ! bProcOption_l( &iArg ) ) return( FALSE );
    }
    else if( osArg == wxT("-d") )
    { // Specify number of lines to display in the text controls
      if( ! bProcOption_d( &iArg ) ) return( FALSE );
    }
    else if( osArg == wxT("-t") )
    { // Specify the temporary file management strategy
      if( ! bProcOption_t( &iArg ) ) return( FALSE );
    }
//  else if( osArg == wxT("-r") )
//  { // Specify a configuration file
//    if( ! bProcOption_r( &iArg ) ) return( FALSE );
//  }
//  else if( osArg == wxT("-c") )
//  { // Specify a SPICE command file
//    if( ! bProcOption_c( &iArg ) ) return( FALSE );
//  }
    else if( osArg == wxT("-s") )
    { // Process simulation engine specifier
      if( ! bProcOption_s( &iArg ) ) return( FALSE );
    }
    else if( osArg == wxT("-a") )
    { // Process analysis type specfier
      if( ! bProcOption_a( &iArg ) ) return( FALSE );
    }
    else if( osArg == wxT("-g") )
    { // Specify Guile procedure for importing a schematic file
      if( ! bProcOption_g( &iArg ) ) return( FALSE );
    }
    else if( osArg.GetChar( 0 ) == wxT('-') )
    { // Invalid option
      std::cout << "Invalid option : " << osArg.mb_str( ) << '\n';
      return( FALSE );
    }
    else break;
  }

  // Process remaining arguments
  if( ! m_poFileTsks->rosGetGuileProc( ).IsEmpty( ) || iArg<m_iArgC )
  {
    if( ! bProcFileName( &iArg ) ) return( FALSE );
    iArg++;
  }

  if( iArg < m_iArgC )
  { // Too many arguments
    os1 << wxT("Too many arguments :");
    for( osArg=m_ppsArgV[ iArg ]; iArg<m_iArgC; osArg=m_ppsArgV[ ++iArg ] )
      os1 << wxT(' ') << osArg;
    std::cout << os1.mb_str( ) << '\n';

    return( FALSE );
  }

  // All's well so display the system banner on the console
  Version( );

  return( TRUE );
}

//*****************************************************************************

