//*****************************************************************************
//                               PrcGnuCap.cpp                                *
//                              ---------------                               *
//  Started     : 01/09/2003                                                  *
//  Last Update : 21/06/2005                                                  *
//  Copyright   : (C) 2003 by M.S.Waters                                      *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "PrcGnuCap.hpp"

//*****************************************************************************
// Constructor.

PrcGnuCap::PrcGnuCap( void ) : PrcSimrBase( )
{
  // Set the simulator binary file name if it can be found
  bSetBinary( wxT("gnucap") );

  m_osInput.Empty( );
}

//*****************************************************************************
// Destructor.

PrcGnuCap::~PrcGnuCap( )
{
}

//*****************************************************************************
// Parse a OPTIONS command and put the data in a simulation object.
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdOPT( wxString & rosCmd, Simulation & roSim )
{
  return( FALSE );
}

//*****************************************************************************
// Parse a GENERATOR command and put the data in a simulation object.
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdGEN( wxString & rosCmd, Simulation & roSim )
{
  return( FALSE );
}

//*****************************************************************************
// Parse a OP command and put the data in a simulation object.
// Eg.s: .OP 27.0
//       .OP 27.0 60.0 BY 1.0
//       .OP 27.0 60.0 TI 1.1
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdOP( wxString & rosCmd, Simulation & roSim )
{
  wxStringTokenizer  ostk1;
  wxString  os1;
  bool      bRtn=TRUE;
  double    df1;

  // Basic argument checks
  ostk1.SetString( rosCmd );
  if( ostk1.CountTokens( ) < 2 )              return( FALSE );

  // Check command type
  os1 = ostk1.GetNextToken( );
  if( os1.Upper( ) != wxT(".OP") )            return( FALSE );

  // Extract the start temperature
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
  if( ! roSim.bSetSwpStart( (float) df1 ) )   bRtn = FALSE;
  if( ! ostk1.HasMoreTokens( ) )
  {
    if( ! roSim.bSetSwpStop( (float) df1 ) )  bRtn = FALSE;
    if( ! roSim.bSetSwpStep( 1.0 ) )          bRtn = FALSE;
    return( TRUE );
  }

  // Extract the stop temperature
  if( ostk1.CountTokens( ) != 3 )             bRtn = FALSE;
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
  if( ! roSim.bSetSwpStop( (float) df1 ) )    bRtn = FALSE;

  // Extract the sweep type: linear or log
  os1 = ostk1.GetNextToken( );
  os1.MakeUpper( );
  if(      os1 == wxT("BY") ) roSim.bSetSwpScale( 0 );
  else if( os1 == wxT("TI") ) roSim.bSetSwpScale( 1 );
  else                                        bRtn = FALSE;

  // Extract the step size
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
  if( ! roSim.bSetSwpStep( (float) df1 ) )    bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Parse a DC command and put the data in a simulation object.
// Eg.s: .DC Vin 0.00 100.00m BY 10.00m TE 27.00
//       .DC Vin 0.00 100.00m TI 1.01 TE 27.00
//       .DC Vin 0.00 100.00m DE 100 TE 27.00
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdDC( wxString & rosCmd, Simulation & roSim )
{
  wxStringTokenizer  ostk1;
  wxString  os1;
  bool      bRtn=TRUE;
  double    df1;
  long      li1;

  // Basic argument checks
  ostk1.SetString( rosCmd );
  if( ostk1.CountTokens( ) != 8 )               return( FALSE );

  // Check command type
  os1 = ostk1.GetNextToken( );
  if( os1.Upper( ) != wxT(".DC") )              return( FALSE );

  // Extract the sweep source label
  os1 = ostk1.GetNextToken( );
  os1 = roSim.rosGetCpnt( os1 );
  if( os1.IsEmpty( ) )                          bRtn = FALSE;
  if( roSim.bSetSrcCpnt( os1 ) )                bRtn = FALSE;

  // Extract the start voltage
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )   bRtn = FALSE;
  if( ! roSim.bSetSwpStart( (float) df1 ) )     bRtn = FALSE;

  // Extract the stop voltage
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )   bRtn = FALSE;
  if( ! roSim.bSetSwpStop( (float) df1 ) )      bRtn = FALSE;

  // Extract the sweep type: linear, log or steps per decade
  os1 = ostk1.GetNextToken( );
  os1.MakeUpper( );
  if(      os1 == wxT("BY") ) roSim.bSetSwpScale( 0 );
  else if( os1 == wxT("TI") ) roSim.bSetSwpScale( 1 );
  else if( os1 == wxT("DE") ) roSim.bSetSwpScale( 2 );
  else                                          bRtn = FALSE;

  // Extract the step size/count
  os1 = ostk1.GetNextToken( );
  if( roSim.iGetSwpScale( ) < 2 )
  {
    if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
    if( ! roSim.bSetSwpStep( (float) df1 ) )    bRtn = FALSE;
  }
  else
  {
    if( ! ConvertType::bStrToInt( os1, &li1 ) ) bRtn = FALSE;
    if( ! roSim.bSetSwpStep( (float) li1 ) )    bRtn = FALSE;
  }

  // Extract the analysis temperature
  os1 = ostk1.GetNextToken( );
  if( os1.Upper( ) == wxT("TE") )
  {
    os1 = ostk1.GetNextToken( );
    if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
    if( ! roSim.bSetTempC( (float) df1 ) )      bRtn = FALSE;
  }

  return( bRtn );
}

//*****************************************************************************
// Parse a AC command and put the data in a simulation object.
// Eg.s: .AC 0.00 100.00K BY 10.00K TE 27.00
//       .AC 0.00 100.00K TI 1.01 TE 30.00
//       .AC 0.00 100.00K DE 1 TE 30.00
//       .AC 0.00 100.00K OC 10 TE 30.00
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdAC( wxString & rosCmd, Simulation & roSim )
{
  wxStringTokenizer  ostk1;
  wxString  os1;
  bool      bRtn=TRUE;
  double    df1;
  long      li1;

  // Basic argument checks
  ostk1.SetString( rosCmd );
  if( ostk1.CountTokens( ) != 7 )               return( FALSE );

  // Check command type
  os1 = ostk1.GetNextToken( );
  if( os1.Upper( ) != wxT(".AC") )              return( FALSE );

  // Extract the start frequency
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )   bRtn = FALSE;
  if( ! roSim.bSetSwpStart( (float) df1 ) )     bRtn = FALSE;

  // Extract the stop frequency
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )   bRtn = FALSE;
  if( ! roSim.bSetSwpStop( (float) df1 ) )      bRtn = FALSE;

  // Extract the sweep type: linear, log or steps per decade
  os1 = ostk1.GetNextToken( );
  os1.MakeUpper( );
  if(      os1 == wxT("BY") ) roSim.bSetSwpScale( 0 );
  else if( os1 == wxT("TI") ) roSim.bSetSwpScale( 1 );
  else if( os1 == wxT("DE") ) roSim.bSetSwpScale( 2 );
  else if( os1 == wxT("OC") ) roSim.bSetSwpScale( 3 );
  else                                          bRtn = FALSE;

  // Extract the step size/count
  os1 = ostk1.GetNextToken( );
  if( roSim.iGetSwpScale( ) < 2 )
  {
    if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
    if( ! roSim.bSetSwpStep( (float) df1 ) )    bRtn = FALSE;
  }
  else
  {
    if( ! ConvertType::bStrToInt( os1, &li1 ) ) bRtn = FALSE;
    if( ! roSim.bSetSwpStep( (float) li1 ) )    bRtn = FALSE;
  }

  // Extract the analysis temperature
  os1 = ostk1.GetNextToken( );
  if( os1.Upper( ) == wxT("TE") )
  {
    os1 = ostk1.GetNextToken( );
    if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
    if( ! roSim.bSetTempC( (float) df1 ) )      bRtn = FALSE;
  }

  return( bRtn );
}

//*****************************************************************************
// Parse a TRANSIENT command and put the data in a simulation object.
// Eg.s: .TR 0.00 100.00m 10.00m TE 27.00 COLD
//       .TR 0.00 100.00m 10.00m TE 27.00 UIC
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdTR( wxString & rosCmd, Simulation & roSim )
{
  wxStringTokenizer  ostk1;
  wxString  os1;
  bool      bRtn=TRUE;
  double    df1;

  // Basic argument checks
  ostk1.SetString( rosCmd );
  if( ostk1.CountTokens( ) != 7 )               return( FALSE );

  // Check command type
  os1 = ostk1.GetNextToken( );
  if( os1.Upper( ) != wxT(".TR") )              return( FALSE );

  // Extract the start time
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )   bRtn = FALSE;
  if( ! roSim.bSetSwpStart( (float) df1 ) )     bRtn = FALSE;

  // Extract the stop time
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )   bRtn = FALSE;
  if( ! roSim.bSetSwpStop( (float) df1 ) )      bRtn = FALSE;

  // Extract the step size
  os1 = ostk1.GetNextToken( );
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )   bRtn = FALSE;
  if( ! roSim.bSetSwpStep( (float) df1 ) )      bRtn = FALSE;

  // Extract the analysis temperature
  os1 = ostk1.GetNextToken( );
  if( os1.Upper( ) == wxT("TE") )
  {
    os1 = ostk1.GetNextToken( );
    if( ! ConvertType::bStrToFlt( os1, &df1 ) ) bRtn = FALSE;
    if( ! roSim.bSetTempC( (float) df1 ) )      bRtn = FALSE;
  }

  // Extract the sweep type: linear, log or steps per decade
  os1 = ostk1.GetNextToken( );
  os1.MakeUpper( );
  if(      os1 == wxT("COLD") ) roSim.bSetSwpScale( 0 );
  else if( os1 == wxT("UIC") )  roSim.bSetSwpScale( 1 );
  else                                          bRtn = FALSE;

  return( bRtn );
}

//*****************************************************************************
// Parse a FOURIER command and put the data in a simulation object.
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdFO( wxString & rosCmd, Simulation & roSim )
{
  return( FALSE );
}

//*****************************************************************************
// Parse a PRINT command and put the data in a simulation object.
//
// Argument List:
//   rosCmd - The command line to be parsed
//   roSim  - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseCmdPR( wxString & rosCmd, Simulation & roSim )
{
  wxStringTokenizer  ostk1;
  wxString  os1;
  bool      bRtn=TRUE;
  size_t    szt1, szt2;

  // Basic argument checks
  ostk1.SetString( rosCmd );
  if( ostk1.CountTokens( ) < 3 )   return( FALSE );

  // Check command type
  os1 = ostk1.GetNextToken( ).Left( 3 );
  if( os1.Upper( ) != wxT(".PR") ) return( FALSE );

  // Extract the analysis type
  os1 = ostk1.GetNextToken( ).Left( 2 );
  os1.MakeUpper( );
  if(      os1 == wxT("OP") ) roSim.bSetAnaType( Simulation::eANA_OP );
  else if( os1 == wxT("DC") ) roSim.bSetAnaType( Simulation::eANA_DC );
  else if( os1 == wxT("AC") ) roSim.bSetAnaType( Simulation::eANA_AC );
  else if( os1 == wxT("TR") ) roSim.bSetAnaType( Simulation::eANA_TR );
  else if( os1 == wxT("FO") ) roSim.bSetAnaType( Simulation::eANA_FO );
  else return( FALSE );

  // Extract the parameters to derive, any complex parts and the test component
  // and/or test node labels
  while( ostk1.HasMoreTokens( ) )
  {
    // Extract the next field
    os1 = ostk1.GetNextToken( );
    if( os1.Length( ) < 4 ) { bRtn = FALSE; continue; }
    // Extract the parameter specifier
    switch( wxToupper( os1.GetChar( 0 ) ) )
    {
      case wxT('V'): roSim.bSetOutPar( Simulation::ePAR_VLT, TRUE ); break;
      case wxT('I'): roSim.bSetOutPar( Simulation::ePAR_CUR, TRUE ); break;
      case wxT('P'): roSim.bSetOutPar( Simulation::ePAR_PWR, TRUE ); break;
      case wxT('R'): roSim.bSetOutPar( Simulation::ePAR_RES, TRUE ); break;
      default: bRtn = FALSE; break;
    }
    // Extract the complex part
    if( roSim.eGetAnaType( ) == Simulation::eANA_AC )
    {
      switch( wxToupper( os1.GetChar( 1 ) ) )
      {
        case wxT('M'): roSim.bSetOutCpx( Simulation::eCPX_MAG,   TRUE ); break;
        case wxT('P'): roSim.bSetOutCpx( Simulation::eCPX_PHASE, TRUE ); break;
        case wxT('R'): roSim.bSetOutCpx( Simulation::eCPX_REAL,  TRUE ); break;
        case wxT('I'): roSim.bSetOutCpx( Simulation::eCPX_IMAG,  TRUE ); break;
        default: bRtn = FALSE; break;
      }
      if( wxToupper( os1.GetChar( 1 )==wxT('M') ) )
        if( os1.Mid( 2, 2 ).Upper( ) == wxT("DB") )
          roSim.bSetOutCpx( Simulation::eCPX_MAGDB, TRUE );
    }
    // Extract the component or node label
    szt1 = os1.Index( wxT('(') );
    szt2 = os1.Index( wxT(')') );
    if( (int)szt1==wxNOT_FOUND || (int)szt2==wxNOT_FOUND || (szt1+1)>=szt2 )
       { bRtn = FALSE; continue; }
    os1 = os1.Mid( szt1+1, szt2-szt1-1 );
    if(      roSim.roasGetNodeLbls( ).Index( os1 ) != wxNOT_FOUND )
      roSim.bAddTstNode( os1 );
    else if( roSim.roasGetCpntLbls( ).Index( os1 ) != wxNOT_FOUND )
      roSim.bAddTstCpnt( os1 );
    else bRtn = FALSE;
  }

  return( bRtn );
}

//*****************************************************************************
// Create the simulator command sequence for an operating point analysis.
//
// Argument List:
//   roSim - Simulation object

void  PrcGnuCap::MakeCmdOP( Simulation & roSim )
{
  wxString  osCmdOP;
  wxString  osCmdPR;
  wxString  osPar, os1;
  uint      uiPar;

  m_osInput = wxT("Temp");

  // Create the OP command
  osCmdOP = wxT(".OP ");
  osCmdOP << roSim.rosGetSwpStart( );
  if( roSim.fGetSwpStop( ) != roSim.fGetSwpStart( ) )
  {
    osCmdOP << wxT(' ') << roSim.rosGetSwpStop( );
    if( roSim.fGetSwpStep( ) != 0.0 )
    {
      switch( roSim.iGetSwpScale( ) )
      {
        case 0:
          osCmdOP << wxT(" BY ");
          osCmdOP << roSim.rosGetSwpStep( );
          break;
        case 1:
          osCmdOP << wxT(" TI ");
          osCmdOP << roSim.rosGetSwpStep( );
          break;
      }
    }
  }

  // Create the PRINT command
  osCmdPR = wxT(".PR OP");
  for( uiPar=Simulation::ePAR_FST; uiPar<=Simulation::ePAR_LST; uiPar++ )
  {
    if( ! roSim.bGetOutPar( (Simulation::eParType) uiPar ) ) continue;

    switch( uiPar )
    {
      case Simulation::ePAR_VLT: osPar = wxT("V("); break;
      case Simulation::ePAR_CUR: osPar = wxT("I("); break;
      case Simulation::ePAR_PWR: osPar = wxT("P("); break;
      case Simulation::ePAR_RES: osPar = wxT("R("); break;
      default: continue;
    }

    // Add any components and/or nodes to derivation list for this parameter
    os1 = rosMakeArgsPR( roSim, osPar );
    if( ! os1.IsEmpty( ) ) osCmdPR << os1;
  }

  // Add simulation commands to Simulation object
  roSim.bAddSimCmd( osCmdPR );
  roSim.bAddSimCmd( osCmdOP );
}

//*****************************************************************************
// Create the simulator command sequence for a DC analysis.
//
// Argument List:
//   roSim - Simulation object

void  PrcGnuCap::MakeCmdDC( Simulation & roSim )
{
  wxString  osCmdPR;
  wxString  osCmdDC;
  wxString  osPar, os1;
  uint      uiPar;

  m_osInput = roSim.rosGetSrcCpnt( );

  // Create the DC command
  osCmdDC = wxT(".DC ");
  osCmdDC << m_osInput << wxT(' ') << roSim.rosGetSwpStart( );
  roSim.bSetSrcCpnt( wxT("") );
  if( roSim.fGetSwpStop( ) != roSim.fGetSwpStart( ) )
  {
    osCmdDC << wxT(' ') << roSim.rosGetSwpStop( );
    if( roSim.fGetSwpStep( ) != 0.0 )
    {
      switch( roSim.iGetSwpScale( ) )
      {
        case 0:
          osCmdDC << wxT(" BY ");
          osCmdDC << roSim.rosGetSwpStep( );
          break;
        case 1:
          osCmdDC << wxT(" TI ");
          osCmdDC << roSim.rosGetSwpStep( );
          break;
        case 2:
          osCmdDC << wxT(" DE ");
          osCmdDC << (int) roSim.fGetSwpStep( );
          break;
      }
    }
  }
  osCmdDC << wxT(" TE ") << roSim.rosGetTempC( );

  // Create the PRINT command
  osCmdPR = wxT(".PR DC");
  for( uiPar=Simulation::ePAR_FST; uiPar<=Simulation::ePAR_LST; uiPar++ )
  {
    if( ! roSim.bGetOutPar( (Simulation::eParType) uiPar ) ) continue;

    switch( uiPar )
    {
      case Simulation::ePAR_VLT: osPar = wxT("V("); break;
      case Simulation::ePAR_CUR: osPar = wxT("I("); break;
      case Simulation::ePAR_PWR: osPar = wxT("P("); break;
      case Simulation::ePAR_RES: osPar = wxT("R("); break;
      default: continue;
    }

    // Add any components and/or nodes to derivation list for this parameter
    os1 = rosMakeArgsPR( roSim, osPar );
    if( ! os1.IsEmpty( ) ) osCmdPR << os1;
  }

  // Add simulation commands to Simulation object
  roSim.bAddSimCmd( osCmdPR );
  roSim.bAddSimCmd( osCmdDC );
}

//*****************************************************************************
// Create the simulator command sequence for a AC analysis.
//
// Argument List:
//   roSim - Simulation object

void  PrcGnuCap::MakeCmdAC( Simulation & roSim )
{
  wxString  osCmdOP;
  wxString  osCmdAC;
  wxString  osCmdPR;
  wxString  osPar, osCpx, os1;
  uint      uiPar, uiCpx;

  m_osInput.Empty( );

  // Create the OP command
  osCmdOP = wxT(".OP ");
  osCmdOP << roSim.rosGetTempC( );

  // Create the AC command
  osCmdAC = wxT(".AC ");
  osCmdAC << roSim.rosGetSwpStart( );
  if( roSim.fGetSwpStop( ) != roSim.fGetSwpStart( ) )
  {
    osCmdAC << wxT(' ') << roSim.rosGetSwpStop( );
    if( roSim.fGetSwpStep( ) != 0.0 )
    {
      switch( roSim.iGetSwpScale( ) )
      {
        case 0:
          osCmdAC << wxT(" BY ");
          osCmdAC << roSim.rosGetSwpStep( );
          break;
        case 1:
          osCmdAC << wxT(" TI ");
          osCmdAC << roSim.rosGetSwpStep( );
          break;
        case 2:
          osCmdAC << wxT(" DE ");
          osCmdAC << (int) roSim.fGetSwpStep( );
          break;
        case 3:
          osCmdAC << wxT(" OC ");
          osCmdAC << (int) roSim.fGetSwpStep( );
          break;
      }
    }
  }
  osCmdAC << wxT(" TE ") << roSim.rosGetTempC( );

  // Create the PRINT commands
  osCmdPR = wxT(".PR AC");
  for( uiPar=Simulation::ePAR_FST; uiPar<=Simulation::ePAR_LST; uiPar++ )
  {
    if( ! roSim.bGetOutPar( (Simulation::eParType) uiPar ) ) continue;

    // Add the parameter prefix
    switch( uiPar )
    {
      case Simulation::ePAR_VLT: osPar = wxT('V'); break;
      case Simulation::ePAR_CUR: osPar = wxT('I'); break;
      case Simulation::ePAR_PWR: osPar = wxT('P'); break;
      case Simulation::ePAR_RES: osPar = wxT('R'); break;
      default: continue;
    }

    // Add the complex part prefix
    for( uiCpx=Simulation::eCPX_FST; uiCpx<Simulation::eCPX_LST; uiCpx++ )
    {
      if( ! roSim.bGetOutCpx( (Simulation::eCpxType) uiCpx ) ) continue;

      osCpx = osPar;

      switch( uiCpx )
      {
        case Simulation::eCPX_MAG:   osCpx << wxT('M'); break;
        case Simulation::eCPX_PHASE: osCpx << wxT('P'); break;
        case Simulation::eCPX_REAL:  osCpx << wxT('R'); break;
        case Simulation::eCPX_IMAG:  osCpx << wxT('I'); break;
        default: continue;
      }
      if( uiCpx != Simulation::eCPX_PHASE )
        if( roSim.bGetOutCpx( Simulation::eCPX_MAGDB ) )
          osCpx << wxT("DB");
      osCpx << '(';

      // Add any components and/or nodes to derivation list for this parameter
      os1 = rosMakeArgsPR( roSim, osCpx );
      if( ! os1.IsEmpty( ) ) osCmdPR << os1;
    }
  }

  // Add simulation commands to Simulation object
  roSim.bAddSimCmd( osCmdPR );
  roSim.bAddSimCmd( osCmdOP );
  roSim.bAddSimCmd( osCmdAC );
}

//*****************************************************************************
// Create the simulator command sequence for a transient analysis.
//
// Argument List:
//   roSim - Simulation object

void  PrcGnuCap::MakeCmdTR( Simulation & roSim )
{
  wxString  osCmdPR;
  wxString  osCmdTR;
  wxString  osPar, os1;
  uint      uiPar;

  m_osInput.Empty( );

  // Create the TR command
  osCmdTR = wxT(".TR ");
  osCmdTR << roSim.rosGetSwpStart( );
  if( roSim.fGetSwpStop( ) != roSim.fGetSwpStart( ) )
  {
    osCmdTR << wxT(' ') << roSim.rosGetSwpStop( );
    osCmdTR << wxT(' ') << roSim.rosGetSwpStep( );
  }
  osCmdTR << wxT(" TE ") << roSim.rosGetTempC( );
  osCmdTR << ( roSim.iGetSwpScale( )==0 ? wxT(" COLD") : wxT(" UIC") );

  // Create the PRINT commands
  osCmdPR = wxT(".PR TR");
  for( uiPar=Simulation::ePAR_FST; uiPar<=Simulation::ePAR_LST; uiPar++ )
  {
    if( ! roSim.bGetOutPar( (Simulation::eParType) uiPar ) ) continue;

    switch( uiPar )
    {
      case Simulation::ePAR_VLT: osPar = wxT("V("); break;
      case Simulation::ePAR_CUR: osPar = wxT("I("); break;
      case Simulation::ePAR_PWR: osPar = wxT("P("); break;
      case Simulation::ePAR_RES: osPar = wxT("R("); break;
      default: continue;
    }

    // Add any components and/or nodes to derivation list for this parameter
    os1 = rosMakeArgsPR( roSim, osPar );
    if( ! os1.IsEmpty( ) ) osCmdPR << os1;
  }

  // Add simulation commands to Simulation object
  roSim.bAddSimCmd( osCmdPR );
  roSim.bAddSimCmd( osCmdTR );
}

//*****************************************************************************
// Create the simulator command sequence for a fourier analysis.
//
// Argument List:
//   roSim - Simulation object

void  PrcGnuCap::MakeCmdFO( Simulation & roSim )
{
  wxString  osCmdPR;
  wxString  osCmdFO;
  wxString  osPar, os1;
  uint      uiPar;

  m_osInput.Empty( );

  // Create the TR command
  osCmdFO = wxT(".FO ");
  osCmdFO << roSim.rosGetSwpStart( );
  if( roSim.fGetSwpStop( ) != roSim.fGetSwpStart( ) )
  {
    osCmdFO << wxT(' ') << roSim.rosGetSwpStop( );
    osCmdFO << wxT(' ') << roSim.rosGetSwpStep( );
  }
  osCmdFO << wxT(" TE ") << roSim.rosGetTempC( );
  osCmdFO << wxT(" COLD");

  // Create the PRINT commands
  osCmdPR = wxT(".PR FO");
  for( uiPar=Simulation::ePAR_FST; uiPar<=Simulation::ePAR_LST; uiPar++ )
  {
    if( ! roSim.bGetOutPar( (Simulation::eParType) uiPar ) ) continue;

    switch( uiPar )
    {
      case Simulation::ePAR_VLT: osPar = wxT("V("); break;
      case Simulation::ePAR_CUR: osPar = wxT("I("); break;
      case Simulation::ePAR_PWR: osPar = wxT("P("); break;
      case Simulation::ePAR_RES: osPar = wxT("R("); break;
      default: continue;
    }

    // Add any components and/or nodes to derivation list for this parameter
    os1 = rosMakeArgsPR( roSim, osPar );
    if( ! os1.IsEmpty( ) ) osCmdPR << os1;
  }

  // Add simulation commands to Simulation object
  roSim.bAddSimCmd( osCmdPR );
  roSim.bAddSimCmd( osCmdFO );
}

//*****************************************************************************
// Generate the simulator commands to be executed based on the simulation
// specifications and the NetList description.
//
// Arguments:
//   roSim - The net list and simulation specification
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bMakeCmds( Simulation & roSim )
{
  switch( roSim.eGetAnaType( ) )
  {
    case Simulation::eANA_OP: MakeCmdOP( roSim ); break;
    case Simulation::eANA_DC: MakeCmdDC( roSim ); break;
    case Simulation::eANA_AC: MakeCmdAC( roSim ); break;
    case Simulation::eANA_TR: MakeCmdTR( roSim ); break;
    case Simulation::eANA_FO: MakeCmdFO( roSim ); break;
    default: return( FALSE );
  }

  if( roSim.roasGetSimCmds( ).IsEmpty( ) ) return( FALSE );

  return( TRUE );
}

//*****************************************************************************
// Create a list of arguments for the Spice PRINT command based on the test
// components and test nodes specified in a Simulation object. A prefix may be
// specified which is prepended to each argument in the list.
//
// Argument List:
//   roSim     - The Simulation object
//   rosPrefix - The prefix to be prepended to each argument
//
// Return Values:
//   Success - A list of correctly formatted arguments
//   Failure - An empty string

wxString & PrcGnuCap::rosMakeArgsPR( Simulation & roSim, wxString & rosPrefix )
{
  static  wxString  osArgs;
  wxChar  oc1;
  size_t  szt1;

  osArgs.Empty( );

  // Add any components to derivation list for this parameter
  for( szt1=0; szt1<roSim.roasGetTstCpnts( ).GetCount( ); szt1++ )
    osArgs << wxT(' ') << rosPrefix << roSim.roasGetTstCpnts( ).Item( szt1 )
           << wxT(')');

  // Add any nodes to derivation list for voltage only
  for( oc1=0, szt1=0; szt1<rosPrefix.Length( ); szt1++ )
  {
    oc1 = rosPrefix.GetChar( szt1 );
    if( ! wxIsspace( oc1 ) ) break;
  }
  if( oc1==wxT('v') || oc1==wxT('V') )
  {
    for( szt1=0; szt1<roSim.roasGetTstNodes( ).GetCount( ); szt1++ )
      osArgs << wxT(' ') << rosPrefix << roSim.roasGetTstNodes( ).Item( szt1 )
             << wxT(')');
  }
  
  return( osArgs );
}

//*****************************************************************************
// Check / format the component lines in the Simulation object so that they are
// compatible with GnuCAP.
//
// Argument List:
//   roasCpnts - The list of component lines
//
// Return Value:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bFmtCpnts( wxArrayString & roasCpnts )
{
  wxStringTokenizer  oStrTok;
  wxString  os1, osNodeC;
  size_t  szt1;

  for( szt1=0; szt1<roasCpnts.GetCount( ); szt1++ )
  {
    oStrTok.SetString( roasCpnts[ szt1 ] );

    os1 = oStrTok.GetNextToken( );

    if( wxToupper( os1.GetChar( 0 ) ) != wxT('Q') ) continue;
    if( oStrTok.CountTokens( ) != 4 )               continue;

    osNodeC = oStrTok.GetNextToken( );

    // Add substrate node to BJT definitions
    os1 << wxT(' ') << osNodeC << wxT(' ') << oStrTok.GetNextToken( )
        << wxT(' ') << oStrTok.GetNextToken( ) << wxT(' ') << osNodeC
        << wxT(' ') << oStrTok.GetNextToken( );

    roasCpnts.RemoveAt( szt1 );
    roasCpnts.Insert( os1, szt1 );
  }

  return( TRUE );
}

//*****************************************************************************
// Format the results file eg. get rid of simulation engine banner message.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bFmtResultsFile( void )
{
  wxString  os1, os2;
  size_t    szt1, szt2;
  wxChar    oc1;

  // Attempt to open the results file
  wxTextFile  oFile( m_oFnmResults.GetFullPath( ) );
  if( ! oFile.Exists( ) ) return( FALSE );
  if( ! oFile.Open( ) )   return( FALSE );

  // Find the beginning of the data area (ie. the last line beginning with '#')
  for( szt1=szt2=0; szt1<oFile.GetLineCount( ); szt1++ )
  {
    os1 = oFile.GetLine( szt1 );
    if( ! os1.IsEmpty( ) )
      if( os1.GetChar( 0 ) == wxT('#') )
        szt2 = szt1;
  }

  // Delete the banner
  for( szt1=0; szt1<szt2; szt1++ ) oFile.RemoveLine( 0 );

  // Insert the input variable name into the column labels
  // Format the column header line
  os1 = oFile.GetFirstLine( );
  if( ! bFmtResultsHead( os1 ) )
    { oFile.Close( ); return( FALSE ); }
  oFile.GetFirstLine( ) = os1;

  // Delete any simulator error messages eg. "open circuit: internal node 3"
  // (All lines should start with a digit except for the first)
  for( szt1=1; !oFile.Eof( ) && szt1<oFile.GetLineCount( ); szt1++ )
  {
    os1 = oFile.GetLine( szt1 );
    if( os1.Length( ) <= 1 )
    { // Delete empty lines
      oFile.RemoveLine( szt1 );
      szt1--;
      continue;
    }
    oc1 = os1.GetChar( 1 );
    if( ! wxIsdigit( oc1 ) )
    { // Delete non-data lines
      oFile.RemoveLine( szt1 );
      szt1--;
      continue;
    }
  }

  // Format data lines
  for( szt1=1; szt1<oFile.GetLineCount( ); szt1++ )
  {
    os1 = oFile.GetLine( szt1 );
    bFmtResultsLine( os1 );
    oFile.GetLine( szt1 ) = os1;
  }

  // Need last line to end with a '\n' or text control will not load it
  if( oFile.GetLastLine( ).Last( ) != wxT('\n') )
    oFile.GetLastLine( ) << wxT('\n');

  oFile.Write( ); // Save the changes to disk
  oFile.Close( ); // Close the file

  return( TRUE );
}

//*****************************************************************************
// Format the column header from the results file.
//
// Argument List:
//   rosLine - The column header to be formatted
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bFmtResultsHead( wxString & rosLine )
{
  wxStringTokenizer  oStrTok;
  wxString  os1;
  int  i1;

  // Tokenize the string
  oStrTok.SetString( rosLine );
  if( oStrTok.CountTokens( ) < 1 ) return( FALSE );

  // Replace the first field if m_osInput is not empty
  os1 = oStrTok.GetNextToken( );
  if( m_osInput.IsEmpty( ) ) rosLine = os1;
  else                       rosLine = (wxString) wxT('#') + m_osInput;

  // Format each field
  while( oStrTok.HasMoreTokens( ) )
  {
    // Place a new field at every 12 columns
    i1 = 12 - (rosLine.Length( ) % 12);
    rosLine.Append( wxT(' '), i1 );
    rosLine << oStrTok.GetNextToken( );
  }

  return( TRUE );
}

//*****************************************************************************
// Format a data line from the results file.
//
// Argument List:
//   rosLine - The data line to be formatted
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bFmtResultsLine( wxString & rosLine )
{
  wxStringTokenizer  oStrTok;
  wxString  os1;
  double    f1;
  int       i1;

  // Tokenize the string
  oStrTok.SetString( rosLine );
  if( oStrTok.CountTokens( ) < 1 ) return( FALSE );

  // Format each field
  rosLine.Empty( );
  while( oStrTok.HasMoreTokens( ) )
  {
    if( rosLine.Length( ) > 0 )
    { // Place a new field at every 12 columns
      i1 = 12 - (rosLine.Length( ) % 12) - 1;
      rosLine.Append( wxT(' '), i1 );
    }
    os1 = oStrTok.GetNextToken( );
    if( ! ConvertType::bStrToFlt( os1, &f1 ) ) f1  =      -9.999e+99;
    if( ! ConvertType::bFltToStr( f1, os1 ) )  os1 = wxT("-9.999e+99");
    rosLine << os1;
  }

  return( TRUE );
}

//*****************************************************************************
// Parse the simulation commands strings found in a Simulation object and load
// the values back into the Simulation object.
//
// Arguments:
//   roSim - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bParseSim( Simulation & roSim )
{
  const wxArrayString & roasCmds=roSim.roasGetSimCmds( );
  wxString  os1;
  size_t    szt1;

  // Set the default state of some of the simulation object attributes
  roSim.bSetOutPar( Simulation::ePAR_VLT,   TRUE );
  roSim.bSetOutCpx( Simulation::eCPX_MAG,   TRUE );
  roSim.bSetOutCpx( Simulation::eCPX_MAGDB, TRUE );

  // Go through the simulation commands and attempt to parse them
  for( szt1=0; szt1<roasCmds.GetCount( ); szt1++ )
  {
    os1 = roasCmds.Item( szt1 );

    if( bParseCmdOPT( os1, roSim ) ) continue;
    if( bParseCmdGEN( os1, roSim ) ) continue;
    if( bParseCmdOP ( os1, roSim ) ) continue;
    if( bParseCmdDC ( os1, roSim ) ) continue;
    if( bParseCmdAC ( os1, roSim ) ) continue;
    if( bParseCmdTR ( os1, roSim ) ) continue;
    if( bParseCmdFO ( os1, roSim ) ) continue;
    if( bParseCmdPR ( os1, roSim ) ) continue;
  }

  return( TRUE );
}

//*****************************************************************************
// Create the simulation commands from the values found in a Simulation object
// and load the command string back into the Simulation object.
//
// Arguments:
//   roSim - The simulation object
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bMakeSim( Simulation & roSim )
{
  wxFileName  ofn1;
  wxString  os1;
  int  i1;

  // Clear error attributes
  m_osError.Empty( );

  // Only execute simulation if it isn't already running
  if( bIsExec( ) )
    { m_osError = wxT("Simulation already running");         return( FALSE ); }

  // Check / format lines for compatibility with GnuCAP
  if( ! bFmtCpnts( roSim.m_oasCpnts ) )
    { m_osError = wxT("Incompatibility with simulator");     return( FALSE ); }

  // Create the simulator commands
  if( ! bMakeCmds( roSim ) )
    { m_osError = wxT("Couldn't create simulator commands"); return( FALSE ); }

  // Save circuit description and SPICE commands to file
  ofn1 = roSim.roGetLoadFile( );
  if( ofn1.GetFullName( ).Find( wxT(".gspiceui.") ) == -1 )
    ofn1.SetName( ofn1.GetName( ) + wxT(".gspiceui") );
  if( ! roSim.bSaveFile( ofn1.GetFullPath( ) ) )
    { m_osError = wxT("Simulation couldn't be saved to file"); return(FALSE); }

  // Construct the command line to execute the simulation
  os1 = wxT("-b ") + roSim.roGetSaveFile( ).GetFullPath( );
  if( ! bSetArgLst( os1.c_str( ) ) )
    { m_osError = wxT("Couldn't set argument list");         return( FALSE ); }

  // Create the results file name
  os1 = roSim.roGetLoadFile( ).GetPath( ) + wxT('/')
      + roSim.roGetLoadFile( ).GetName( );
  i1 = os1.Find( wxT(".gspiceui") );
  if( i1 > 0 ) os1 = os1.Truncate( (size_t) i1 );
  os1 << wxT('.') << roGetBinary( ).GetName( );
  switch( roSim.eGetAnaType( ) )
  {
    case Simulation::eANA_OP: os1 << wxT(".op"); break;
    case Simulation::eANA_DC: os1 << wxT(".dc"); break;
    case Simulation::eANA_AC: os1 << wxT(".ac"); break;
    case Simulation::eANA_TR: os1 << wxT(".tr"); break;
    case Simulation::eANA_FO: os1 << wxT(".fo"); break;
    default: return( FALSE );
  }
  if( ! bSetResultsFile( os1.c_str( ) ) )
    { m_osError = wxT("Couldn't set results file name");     return( FALSE ); }

  return( TRUE );
}

//*****************************************************************************
// Check for error messages in the simulator output ???

//  bData = FALSE;
//  for( os1=oFileCct.GetFirstLine(); !oFileCct.Eof(); os1=oFileCct.GetNextLine())
//  {
//    if( os1.Length( ) <= 1 ) continue;
//    oc1 = os1.GetChar( 0 );
//    if( bData )
//    { // Look for simulator error messages
//      if( !wxIsdigit( oc1 ) && oc1!=wxT(' ') && oc1!=wxT('-') )
//      { // Simulation error message found
//        osError = wxT("Error/s encountered running simulation");
//        break;
//      }
//    }
//    else
//    { // Look for the beginning of the simulation data
//      if( oc1 == wxT('#') ) bData = TRUE;
//    }
//  }

//*****************************************************************************
// Execute a simulation.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  PrcGnuCap::bExec( void )
{
  wxString  os1, os2;

  // Execute the simulation
  if( ! PrcBase::bExec( ) )      return( FALSE );

  // Save the simulation results to the log file
  os1 = roGetLogFile( ).GetFullPath( );
  if( ! bSaveOutput( os1 ) )     return( FALSE );

  // Copy the log file to the results file
  os2 = roGetResultsFile( ).GetFullPath( );
  if( ! wxCopyFile( os1, os2 ) ) return( FALSE );

  // Format the simulation results
  if( ! bFmtResultsFile( ) )     return( FALSE );

  return( TRUE );
}

//*****************************************************************************
